SystemOrganization addCategory: #MonadTests!

TestCase subclass: #MonadTestCase
	instanceVariableNames: 'monad sampleValue sampleBlock f g'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MonadTests'!

!MonadTestCase commentStamp: 'fbs 7/6/2011 07:27' prior: 0!
A base test case for monads. You need to implement
* #sampleValues, a sequenceable collection of non-monadic values;
* #sampleBlock, a unary block that returns a monadic value;
* #f, g, both unary blocks that return monadic values.!

!MonadTestCase class methodsFor: 'testing' stamp: 'fbs 7/5/2011 17:54'!
isAbstract
	^ self name == #MonadTestCase.! !

!MonadTestCase methodsFor: 'utilities' stamp: 'fbs 7/6/2011 07:38'!
assert: expected equals: received description: prefixString
	self
		assert: expected = received
		description: ('{1}: {2} but was {3}' format: {prefixString. expected printString. received printString.}).! !

!MonadTestCase methodsFor: 'utilities' stamp: 'fbs 7/6/2011 07:34'!
assertAssociative: m with: f and: g
	self
		assert: ((m >>= f) >>= g) equals: (m >>= [:x | (f value: x) >>= g])
		description: 'Associativity'! !

!MonadTestCase methodsFor: 'utilities' stamp: 'fbs 7/6/2011 07:52'!
assertLeftIdentity: aValue with: aUnaryMonadicBlock in: aMonad
	self
		assert: ((aMonad return: aValue) >>= aUnaryMonadicBlock)
		equals: (aUnaryMonadicBlock value: aValue)
		description: 'Left identity'.! !

!MonadTestCase methodsFor: 'utilities' stamp: 'fbs 7/6/2011 07:56'!
assertRightIdentity: aValue with: aUnaryMonadicBlock in: aMonad
	| monadicValue |
	monadicValue := self sampleBlock value: aValue.
	self
		assert: (monadicValue >>= aMonad return)
		equals: monadicValue
		description: 'Left identity'.! !

!MonadTestCase methodsFor: 'data' stamp: 'fbs 7/6/2011 07:28'!
f
	"Return a unary block that returns a monadic value, for testing associativity."
	self subclassResponsibility.! !

!MonadTestCase methodsFor: 'data' stamp: 'fbs 7/6/2011 07:28'!
g
	"Return a unary block that returns a monadic value, for testing associativity."
	self subclassResponsibility.! !

!MonadTestCase methodsFor: 'data' stamp: 'fbs 7/6/2011 07:48'!
monad
	"Return the monad-forming class."
	self subclassResponsibility.! !

!MonadTestCase methodsFor: 'data' stamp: 'fbs 7/6/2011 07:27'!
monadicValue
	"Some representative monadic value."
	self sampleBlock value: self sampleValues first.! !

!MonadTestCase methodsFor: 'data' stamp: 'fbs 7/6/2011 07:28'!
sampleBlock
	"Return a unary block that returns a monadic value."
	self subclassResponsibility.! !

!MonadTestCase methodsFor: 'data' stamp: 'fbs 7/6/2011 07:25'!
sampleValues
	self subclassResponsibility.! !

!MonadTestCase methodsFor: 'testing - monad' stamp: 'fbs 7/6/2011 07:49'!
testMonadObeysAssociativeRule
	self sampleValues do:
		[:each | | monadicValue |
		monadicValue := self sampleBlock value: each.
		self assertAssociative: monadicValue with: self f and: self g].! !

!MonadTestCase methodsFor: 'testing - monad' stamp: 'fbs 7/6/2011 07:57'!
testMonadObeysLeftIdentity
	self sampleValues do:
		[:each | self assertLeftIdentity: each with: self sampleBlock in: self monad].! !

!MonadTestCase methodsFor: 'testing - monad' stamp: 'fbs 7/6/2011 07:56'!
testMonadObeysRightIdentity
	self sampleValues do:
		[:each | self assertRightIdentity: each with: self sampleBlock in: self monad].! !
